;------------------------------------------------------------
;
; Lampensteuerung
; Christian Finger 1999
;
;------------------------------------------------------------

LIST  p=16C84            ; Prozessortyp
;
; Anschlsse :

; PA0   = Klappenschalter (1=Auf)
; PA1   = Schalter Armaturenbrett
; PB0   = Lampentransistor (1=Einschalten)

;--------------- Definitionen --------------------

; Darstellung fr Zielregister

w                            equ     h'0000'
f                            equ     h'0001'

;----- Register ----------------------------------

status                       equ     h'0003'    ; Statusregister
port_a                       equ     h'0005'    ; IO
port_b                       equ     h'0006'    ; IO

option_reg                   equ     h'0081'    ; Option-Register
trisa                        equ     h'0085'    ; IO-Richtung
trisb                        equ     h'0086'    ; IO-Richtung

;---------- EEprom-Funktionen --------------------------
rp0     equ     5       ; Page-Register 0
rp1     equ     6       ; Page-Register 1
eedata  equ     8       ; EEprom data register     Page 0!
eeadr   equ     9       ; EEprom adress register   Page 0!
eecon1  equ     08      ; EEprom control register1 Page 1!
rd      equ     0       ; RD=1 enables 1 EEPROM read (reset auto.)
wr      equ     1       ;
wren    equ     2       ;
wrerr   equ     3       ;
eeif    equ     4       ; EEPROM-Write complete reset by software
eecon2  equ     09      ; control register2        Page 1!


; Interrupt-Kontrolle
intcon  equ     0B      ; Interrupt-Control-Register
rbif    equ     0       ; Port-Change-Flag fuer RB<7:4> Reset in Software
intf    equ     1       ; Interrupt-Flag, set if int
rtif    equ     2       ; RTCC-Overflow
rbie    equ     3       ; RBIF Enable 1=enable
inte    equ     4       ; Interrupt enable fuer INTF
rtie    equ     5       ; Interrupt enable fuer RTIF
eeie    equ     6       ; Eeprom write interrupt enbale bit
gie     equ     7       ; Global interrupt enable
dc      equ     1       ; Digital Carry fuer ADDWF...
pd      equ     3       ; Power down-Bit


;----- STATUS Bits -----------------------------------------------------

#define z       status,2   ; Zero-Flag
#define c       status,0   ; Carry-Flag

;----- Zuweisungen -----------------------------------------------------

ORG     0CH             ; Variablenplaetze reservieren


flags           RES 1              ; Flagregister
zeitzaehler_lsb RES 1              ; LSB des Durchlaufzaehlers
zeitzaehler_msb RES 1              ; MSB des Durchlaufzaehlers

; gespeicherte Statuswerte, jeweils das erste Bit gibt den letzen Wert an
; Liegen in dieser Reihenfolge auch im EEprom (beginnt ab Adresse Null)
altklappe       RES 1              ; Vorheriger Zustand der Heckklappe
altschalter     RES 1              ; Vorheriger Zustand der Heckklappe
altlampe        RES 1              ; Vorheriger Zustand der Lampe

; Wartezhler
delay_count1    RES 1              ; EIn Schleifenzhler halt...
delay_count2    RES 1              ; Ein Schleifenzhler halt...

; EEprom-Funktionen
byte             RES   1       ; Zum schreiben und lesen des EEproms
adresse          RES   1       ; Aktuelle Adresse

; Verschiedenes

temp1            RES  1
temp2            RES  1

;----- Flagdefinitionen ----------------------

#define lampenflag flags,0         ; Status der Lampe
#define aenderung  flags,1         ; nderung an den Schaltern da ?
#define klappe     flags,2         ; Aktueller Status der Heckklappe (1=Auf)
#define zeitflag   flags,2         ; Zeit abgelaufen = 1


;*********************************************************

org 0x00    ; Starteinsprung
goto start  ; Programmeinsprung

;*********************************************************

;------------------ Unterprogramme ------------------------------

;***********************************************************************
; Read from EEPROM Addresse in "adresse" -> Wert in "byte" zurueck
; Changed: W
; 0 Stacklevel
;***********************************************************************
read_eep

         movf    adresse,w      ; adresse in w
         movwf   eeadr          ; w in Adresse schieben
         bsf     status,rp0     ; in Page 1 schalten
         bsf     eecon1,rd      ; Read enable auf 1 (automatisch zurueck)

         bcf     status,rp0     ;
         movf    eedata,w       ; in Page 0 schalten
         movwf   byte           ; und in byte schreiben
         return

;***********************************************************************
; Write to EEPROM, Datum in "byte", Addresse in "adresse"
; Changed: byte,W
; 0 Stacklevel
;***********************************************************************
write_eeprom
        movf adresse,w          ; adresse in w
        movwf   eeadr           ; In Adressregister schreiben
        movf    byte,w          ; zuruecksichern und
        movwf   eedata          ; Data schreiben
        movf    intcon,w        ; Interrupt-Control-Register sichern
        movwf   byte
        bcf     intcon,gie      ; Interrupts global disablen
        bsf     status,rp0      ; auf Page 1 umschalten
        bsf     eecon1,wren     ; Writeenable auf 1
        movlw   H'55'           ; Schreiben
        movwf   eecon2          ; Page 1
        movlw   H'AA'
        movwf   eecon2
        bsf     eecon1,wr       ; set write bit
loop13  btfsc   eecon1,wr       ; schreiben fertig?
        goto    loop13          ; nein, dann warten...
        bcf     eecon1,wren     ; write enable off
        bcf     status,rp0      ; wieder auf Page 0
        btfsc   byte,gie        ; war Interrupt enabled?
        bsf     intcon,gie      ; wenn ja, dann wieder ein, sonst nicht
        retlw   0               ; wenn Byte geschrieben wurde, dann mit 0 ausst.

;****************************************************************
; Ldt den Zeitzhler vor fr die Anzahl der Schleifendurchlufe
; 3a98h fr 100 KHz Takt
;****************************************************************

lade_zaehler  movlw 0x98
              movwf zeitzaehler_lsb
              movlw 0x3a
              movwf zeitzaehler_msb
	      return                          ; Und raus


;*******************************************************************
; Erniedrigt den Zeitzhler und setzt ggf. das Ablaufflag
;*******************************************************************
dec_zeitzaehler
                bcf zeitflag                ; Zeitflag rcksetzen
                decf zeitzaehler_lsb,f      ; LSB -1
                movf zeitzaehler_lsb,w      ; LSB in w
                sublw 0xff                  ; bilde w:=w-255
                btfss z                     ; Hat berlauf stattgefunden ?
                goto dec_ende               ; Wenn nicht -> dec_ende
                decf zeitzaehler_msb,f      ; berlauf, MSB-1
                
                ; Jetzt testen, ob beide Null sind
dec_ende        movf zeitzaehler_lsb,f      ; LSB schieben, setzt Z-Flag
                btfsc z                     ; War lsb schon Null ?
                goto dec_return             ; Wenn nicht -> Ende

                movf zeitzaehler_msb,f      ; MSB schieben, setzt Z-Flag
                btfsc z                     ; War msb schon Null ?
                bsf zeitflag                ; Wenn ja -> Zeit ist abgelaufen
dec_return      return                      ; Und raus

;************************
; Schaltet die Lampe ein
;************************
einschalten
            bsf port_b,0        ; Es werde Licht.......
	    return              ; Und raus

;************************
; Schaltet die Lampe aus
;************************
ausschalten
            bcf port_b,0        ; Und es ward finstere Nacht.......
	    return              ; Und raus

;*******************************************************************
; Holt den Status der Heckklappe rein (Port_A,0)
; Setzt die Flags AENDERUNG wenn ein neuer Zustand eingetreten ist
; und dann das Flag KLAPPE (1=Auf)
;*******************************************************************

get_klappe  bcf aenderung        ; Keine nderung....

            movf port_a,w        ; Port A in w holen
            movwf temp1          ; w in temp1 schieben
            movlw 0x01           ; 1 in w
            andwf temp1,f        ; w mit temp1 AND-Verkpfen

            call delay           ; 5 ms Pause machen...

            movf port_a,w        ; Port A in w holen
            movwf temp2          ; w in temp1 schieben
            movlw 0x01           ; 1 in w
            andwf temp2,f        ; w mit temp2 AND-Verkpfen

            ; Jetzt muss temp1=temp2 sein, sonst wahrscheinlich Strung
            ; oder Prellen
            movf temp1,w        ; Temp1 in w schieben
            subwf temp2,w       ; Temp2 davon abziehen
            btfsc z             ; Waren sie Gleich, d.h. Ergebniss 0 ?
            goto klappe_ret     ; Wenn nicht -> Raus

            ; Hier gelandet ist ein stabiler Zustand da, jetzt vergleichen
            ; ob eine nderung stattgefunden hat
            movf temp1,w       ; temp1 in w schieben
            subwf altklappe,w  ; w von Altklappe abziehen

            ; SInd die beiden Werte jetzt unterschiedlich, hat eine
            ; nderung stattgefunden
            btfss z
            goto klappe_ret   ; Hat sich nix getan, und weg hier....

            ; Die nderung ist da, ein Neuer ist an der Macht, die Massen
            ; jubeln....
            bsf aenderung     ; Flag fr nderung setzen
            movf temp1,w      ; temp1 in w
            movwf altklappe   ; Status sichern

            ; altklappe in EEprom sichern
            movwf byte        ; w in byte schieben
            movlw 0x00        ; 0 in w schieben
            movwf adresse     ; w in adresse schieben
            call write_eeprom ; Und speichern

            bcf klappe        ; Zuerst isse dicht....
            btfsc altklappe,0 ; Ist das Bit gesetzt ?
            bsf klappe        ; Wenn ja ist die Klappe auf...

klappe_ret  return              ; Und raus

;*******************************************************************
; Holt den Status des Schalters rein und setzt das Flags AENDERUNG,
; wenn ein neuer Zustand eingetreten ist (Port_a,1)
;*******************************************************************

get_schalter
            bcf aenderung        ; Keine nderung....

            movf port_a,w        ; Port A in w holen
            movwf temp1          ; w in temp1 schieben
            movlw 0x02           ; 2 in w
            andwf temp1,f        ; w mit temp1 AND-Verkpfen

            call delay           ; 5 ms Pause machen...

            movf port_a,w        ; Port A in w holen
            movwf temp2          ; w in temp1 schieben
            movlw 0x02           ; 2 in w
            andwf temp2,f        ; w mit temp2 AND-Verkpfen

            ; Jetzt muss temp1=temp2 sein, sonst wahrscheinlich Strung
            ; oder Prellen
            movf temp1,w        ; Temp1 in w schieben
            subwf temp2,w       ; Temp2 davon abziehen
            btfsc z             ; Waren sie Gleich, d.h. Ergebniss 0 ?
            goto schalter_ret   ; Wenn nicht -> Raus

            ; Hier gelandet ist ein stabiler Zustand da, jetzt vergleichen
            ; ob eine nderung stattgefunden hat
            movf temp1,w         ; temp1 in w schieben
            subwf altschalter,w  ; w von Altschalter abziehen

            ; SInd die beiden Werte jetzt unterschiedlich, hat eine
            ; nderung stattgefunden
            btfss z
            goto schalter_ret   ; Hat sich nix getan, und weg hier....

            ; Die nderung ist da, ein Neuer ist an der Macht, die Massen
            ; jubeln....
            bsf aenderung     ; Flag fr nderung setzen
            clrf altschalter  ; Altschalter erst einmal Null
            btfsc temp1,1     ; Ist der Schalter=1 ?
            bsf altschalter,0 ; Wenn ja, Bit 0 setzen

            ; altschalter in EEprom sichern
            movwf byte        ; w in byte schieben
            movlw 0x01        ; 0 in w schieben
            movwf adresse     ; w in adresse schieben
            call write_eeprom ; Und speichern

schalter_ret  return              ; Und raus


;*******************************************************************
; Macht ca. 20 ms Pause
;*******************************************************************
delay       movlw 0xff            ; 255 i w
            movwf delay_count1    ; w in  delay_count1

delay_innen decfsz delay_count1,f ; delay_count1-1
            goto delay_innen      ; Nocht nicht Null ? -> Weiter

            clrwdt                ; Watchdog befriedigen

	    return                ; Und raus

;*******************************************************************
; Macht ein paar grundlegende Dinge zu Beginn der Geschichte
;*******************************************************************
init        ; Richtungsregister setzen
            movlw   B'11111111'     ; Port A 0=out 1=in
            tris    port_a          ; setzen
            movlw   B'00000000'     ; 0=Output, 1= input
            tris    port_b          ; Portrichtungsregister setzen (Port B)
	    return                ; Und raus

;*******************************************************************
; Holt den zuletzt gespeicherten Zustand rein
;*******************************************************************
get_flags                         ; Alles aus EEProm holen

            movlw 0x00            ; 0 in w schieben
            movwf adresse         ; w in adresse schieben
            call read_eep         ; Daten aus eeprom holen
            movf byte,w           ; byte in w schieben
            movwf altklappe       ; Und Wert herstellen

            movlw 0x01            ; 1 in w schieben
            movwf adresse         ; w in adresse schieben
            call read_eep         ; Daten aus eeprom holen
            movf byte,w           ; byte in w schieben
            movwf altschalter     ; Und Wert herstellen

            movlw 0x02            ; 1 in w schieben
            movwf adresse         ; w in adresse schieben
            call read_eep         ; Daten aus eeprom holen
            movf byte,w           ; byte in w schieben
            movwf altlampe        ; Und Wert herstellen

            ; Lampenflag wiederherstellen
            bcf lampenflag        ; Erst mal isse aus....
            btfsc altlampe,0      ; Ist Bit 0 gesetzt ?
            bsf lampenflag        ; Wenn ja, ist die Lampe an...

            ; Klappenflag wiederherstellen
            bcf klappe            ; Erst mal isse zu....
            btfsc altklappe,0     ; Ist Bit 0 gesetzt ?
            bsf klappe            ; Wenn ja, ist die Klappe auf

	    return                ; Und raus


;************************ Hauptprogramm *********************************

start
              call init                             ; EIn paar Dinge erledigen
              call lade_zaehler                     ; Ldt den Zeitzhler vor
              call get_flags                        ; Die letzen Flags reinholen

prog_start    btfsc lampenflag                      ; Soll Lampe an oder aus sein ?
              goto lampe_ein                        ; -> Lampe anschalten

lampe_aus     call ausschalten                      ; Lampe ausschalten
              call lade_zaehler                     ; Ldt den Zeitzhler vor
              goto schalter_test                    ; Weiter mit Schalterabfrage

lampe_ein     call einschalten                      ; Lampe einschalten
              call dec_zeitzaehler                  ; Zeitzaehler erniedrigen und vergleichen
              btfss zeitflag                        ; Zeit schon abgelaufen ?
              goto klappen_test                     ; Wenn nicht, Heckklappe abfragen

              ; Hier gelandet ist die Zeit abgelaufen, die Lampe soll ausgehen
              bcf lampenflag                        ; Lampenflag rcksetzen

              ; Lampenflag in eeprom als altlampe
              clrf byte                             ; Byte 0 setzen
              movlw 0x02                            ; 2 in w
              movwf adresse                         ; w in Adresse
              call write_eeprom                     ; und speichern

              goto prog_start                       ; Und wieder von vorne...

klappen_test  call get_klappe                     ; Status der Heckklappe reinholen
              btfss aenderung                     ; Hat sich der Status gendert ?
              goto schalter_test                  ; Nicht ? Dann Schalter checken

              btfss klappe                        ; Wurde die Klappe geffnet ?
              goto klappe_zu                      ; Wenn nicht -> Klappe dicht

klappe_auf    bsf lampenflag                      ; Lampe an....

              ; Lampenflag in eeprom als altlampe
              movlw 0x01                            ; 1 in w
              movwf byte                            ; w in byte schieben
              movlw 0x02                            ; 2 in w
              movwf adresse                         ; w in Adresse
              call write_eeprom                     ; und speichern

              goto schalter_test                  ; Und Schalter abfragen

klappe_zu     bcf lampenflag                      ; Lampe aus....
              ; Lampenflag in eeprom als altlampe
              clrf byte                             ; Byte 0 setzen
              movlw 0x02                            ; 2 in w
              movwf adresse                         ; w in Adresse
              call write_eeprom                     ; und speichern
              goto schalter_test                  ; Und Schalter abfragen

schalter_test call get_schalter                  ; Schalterstatus abfragen
              btfss aenderung                    ; Hat sich was getan ?
              goto prog_start                    ; Nein ? Dann weiter...

schalter      btfsc lampenflag                   ; War sie an ?
              goto schalter_aus                  ; WEnn ja, -> schalter_aus

schalter_ein  bsf lampenflag                     ; Lampe ein...
              goto prog_start                    ; Und weiter...

schalter_aus  bcf lampenflag                     ; Lampe aus...
              ; Lampenflag in eeprom als altlampe
              clrf byte                             ; Byte 0 setzen
              movlw 0x02                            ; 2 in w
              movwf adresse                         ; w in Adresse
              call write_eeprom                     ; und speichern

              goto prog_start                    ; Und weiter...

 end ; Und hier isses vorbai, neeee !
